package com.softslate.commerce.administrator.administrator;

import java.util.Arrays;
import java.util.Collection;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.beanutils.BeanUtils;
import org.apache.commons.beanutils.PropertyUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;

import com.softslate.commerce.businessobjects.administrator.Role;
import com.softslate.commerce.businessobjects.administrator.RoleProcessor;
import com.softslate.commerce.customer.core.BaseAction;

/**
 * Struts action class corresponding to requests to
 * "/administrator/RoleEditForm.do". Loads in the current values of the record
 * being edited and populates a <code>RoleAddEditForm</code> object.
 * 
 * @author David Tobey
 * 
 */
public class RoleEditFormAction extends BaseAction {

	static Log log = LogFactory.getLog(RoleAddFormAction.class);

	public ActionForward execute(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response)
			throws Exception {
		if (log.isDebugEnabled())
			log.debug("Starting action.");
		RoleAddEditForm baseForm = (RoleAddEditForm) form;
		baseForm.initializeProperties(mapping, request);

		// Get the record to edit
		RoleProcessor processor = (RoleProcessor) baseForm
				.getBusinessObjectFactory().createObject(
						"roleProcessorImplementer");
		Role role = processor.loadRoleFromID(PropertyUtils.describe(form));
		BeanUtils.copyProperties(baseForm, role);

		// Enforce role permissions
		if (!request.isUserInRole("superuser")
				&& !request.isUserInRole("administrator")
				&& !request.isUserInRole("administratorEdit")) {
			request.setAttribute("editPermission", "false");
		} else {
			request.setAttribute("editPermission", "true");
		}

		// Create breadcrumbing collection
		Collection bcs = Arrays.asList(new String[] { "Home", "Roles",
				"Role Details" });
		request.setAttribute("breadcrumbs", bcs);

		request.setAttribute("role", role);
		request.setAttribute("roleAddEditForm", baseForm);
		return mapping.findForward("success");
	}
}
