package com.softslate.commerce.administrator.product;

import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.beanutils.PropertyUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.action.ActionMessage;
import org.apache.struts.action.ActionMessages;

import com.softslate.commerce.businessobjects.product.Attribute;
import com.softslate.commerce.businessobjects.product.AttributeProcessor;
import com.softslate.commerce.customer.core.ActionUtils;
import com.softslate.commerce.customer.core.BaseAction;

/**
 * Struts action class that corresponds to requests to
 * "/administrator/AttributeEdit.do". Processes a request to edit a record
 * within the system.
 * 
 * @author David Tobey
 * 
 */
public class AttributeEditAction extends BaseAction {

	static Log log = LogFactory.getLog(AttributeEditAction.class);

	public ActionForward execute(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response)
			throws Exception {
		if (log.isDebugEnabled())
			log.debug("Starting action.");

		AttributeAddEditForm baseForm = (AttributeAddEditForm) form;

		// Handle generating the code from the name if it is empty
		ActionUtils actionUtils = (ActionUtils) baseForm
				.getBusinessObjectFactory().createUtilsObject(
						"actionUtilsImplementer");
		if ((baseForm.getAttributeCode() == null || baseForm.getAttributeCode()
				.equals(""))
				&& (baseForm.getAttributeName() != null || !baseForm
						.getAttributeName().equals(""))) {
			baseForm.setAttributeCode(actionUtils.generateCodeFromName(baseForm
					.getAttributeName()));
		}

		// Edit the record
		AttributeProcessor processor = (AttributeProcessor) baseForm
				.getBusinessObjectFactory().createObject(
						"attributeProcessorImplementer");
		Map results = processor.editAttribute(PropertyUtils.describe(form));

		// Successful insert
		if (results == null || results.get("resultCode") == null
				|| results.get("resultCode").equals("0")) {

			// Set the success messages
			baseForm.getMessages().add(ActionMessages.GLOBAL_MESSAGE,
					new ActionMessage("messages.recordSaved"));
			saveMessages(baseForm.getRequest(), baseForm.getMessages());

			return mapping.findForward("success");
		}

		// Set the error messages
		if (results != null && results.get("existingRecord") != null) {
			Attribute existingRecord = (Attribute) results
					.get("existingRecord");
			String existingCode = existingRecord.getAttributeCode();
			baseForm.getErrors().add(
					ActionMessages.GLOBAL_MESSAGE,
					new ActionMessage("errors.resultCode"
							+ results.get("resultCode"), results
							.get("duplicateField"), existingCode));
			baseForm.getErrors().add(
					ActionMessages.GLOBAL_MESSAGE,
					new ActionMessage("errors.editExistingRecord",
							"AttributeEditForm.do?attributeID="
									+ existingRecord.getAttributeID(), results
									.get("duplicateField"), existingCode));
			saveErrors(baseForm.getRequest(),
					(ActionMessages) baseForm.getErrors());
		}

		return mapping.findForward("success");
	}
}
