package com.softslate.commerce.administrator.product;

import java.util.Arrays;
import java.util.Collection;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.beanutils.BeanUtils;
import org.apache.commons.beanutils.PropertyUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;

import com.softslate.commerce.businessobjects.core.BusinessObjectFactory;
import com.softslate.commerce.businessobjects.product.Attribute;
import com.softslate.commerce.businessobjects.product.AttributeProcessor;
import com.softslate.commerce.businessobjects.product.Option;
import com.softslate.commerce.businessobjects.product.OptionProcessor;
import com.softslate.commerce.businessobjects.product.Product;
import com.softslate.commerce.businessobjects.product.ProductProcessor;
import com.softslate.commerce.customer.core.BaseAction;

/**
 * Struts action class corresponding to requests to
 * "/administrator/OptionEditForm.do". Loads in the current values of the record
 * being edited and populates a <code>OptionAddEditForm</code> object.
 * 
 * @author David Tobey
 * 
 */
public class OptionEditFormAction extends BaseAction {

	static Log log = LogFactory.getLog(OptionEditFormAction.class);

	public ActionForward execute(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response)
			throws Exception {
		if (log.isDebugEnabled())
			log.debug("Starting action.");
		OptionAddEditForm baseForm = (OptionAddEditForm) form;
		baseForm.initializeProperties(mapping, request);

		// Get the record to edit
		OptionProcessor processor = (OptionProcessor) baseForm
				.getBusinessObjectFactory().createObject(
						"optionProcessorImplementer");
		Option option = processor
				.loadOptionFromID(PropertyUtils.describe(form));
		BeanUtils.copyProperties(baseForm, option);

		// Load the parent and grandparent if an id is present
		BusinessObjectFactory bof = (BusinessObjectFactory) request
				.getAttribute("businessObjectFactory");
		Attribute attribute = null;
		if (log.isDebugEnabled())
			log.debug("Checking for parent: " + baseForm.getAttributeID());
		if (baseForm.getAttributeID() != null
				&& baseForm.getAttributeID().length() > 0) {
			AttributeProcessor parentProcessor =  bof
					.createObject(AttributeProcessor.class);
			attribute = parentProcessor.loadAttributeFromID(PropertyUtils
					.describe(baseForm));
			request.setAttribute("attribute", attribute);
		}
		Product product = null;
		if (log.isDebugEnabled())
			log.debug("Checking for parent: " + baseForm.getProductID());
		if (baseForm.getProductID() != null
				&& baseForm.getProductID().length() > 0) {
			ProductProcessor gpProcessor =  bof
					.createObject(ProductProcessor.class);
			product = gpProcessor.loadProductFromID(PropertyUtils
					.describe(baseForm));
			request.setAttribute("product", product);
		}

		// Enforce role permissions
		if (!request.isUserInRole("superuser")
				&& !request.isUserInRole("administrator")
				&& !request.isUserInRole("optionEdit")) {
			request.setAttribute("editPermission", "false");
		} else {
			request.setAttribute("editPermission", "true");
		}

		// Navigation
		if (baseForm.getProductID() != null
				&& baseForm.getProductID().length() > 0) {

			// Create breadcrumbing collection
			Collection bcs = Arrays.asList(new String[] { "Home", "Products",
					"Product Details", "Attributes", "Attribute Details",
					"Options", "Option Details" });
			request.setAttribute("breadcrumbs", bcs);

			// Create childSubNavigation collection
			Collection childsn = Arrays.asList(new String[] {
					"Attribute Details", "Options", "Add Option",
					"Option Details" });
			request.setAttribute("childSubNavigation", childsn);

		} else {

			// Create breadcrumbing collection
			Collection bcs = Arrays.asList(new String[] { "Home", "Attributes",
					"Attribute Details", "Options", "Option Details" });
			request.setAttribute("breadcrumbs", bcs);
		}

		request.setAttribute("option", option);
		request.setAttribute("optionAddEditForm", baseForm);
		return mapping.findForward("success");
	}
}
