package com.softslate.commerce.administrator.product;

import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.beanutils.PropertyUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.action.ActionMessage;
import org.apache.struts.action.ActionMessages;

import com.softslate.commerce.businessobjects.product.Product;
import com.softslate.commerce.businessobjects.product.ProductProcessor;
import com.softslate.commerce.customer.core.ActionUtils;

/**
 * Struts action class that corresponds to requests to "/administrator/ProductAdd.do". Processes a request to add a
 * record to the system.
 * 
 * @author David Tobey
 * 
 */
public class ProductAddAction extends ProductEditAction {

	static Log log = LogFactory.getLog(ProductAddAction.class);

	public ActionForward execute(ActionMapping mapping, ActionForm form, HttpServletRequest request,
			HttpServletResponse response) throws Exception {
		if (log.isDebugEnabled())
			log.debug("Starting action.");

		// Add the record
		ProductAddEditForm baseForm = (ProductAddEditForm) form;

		// Try uploading any files, these will throw exceptions if they do not
		// work therefore the product will not be added in that case.
		processImages(baseForm);

		// Handle generating the code from the name if it is empty
		if ((baseForm.getCode() == null || baseForm.getCode().equals(""))
				&& (baseForm.getName() != null || !baseForm.getName().equals(""))) {
			ActionUtils actionUtils = (ActionUtils) baseForm.getBusinessObjectFactory().createUtilsObject(
					"actionUtilsImplementer");
			baseForm.setCode(actionUtils.generateCodeFromName(baseForm.getName()));
		}

		// Handle setting the seoCode field to null if it is empty
		if (baseForm.getSeoCode() != null && baseForm.getSeoCode().equals("")) {
			baseForm.setSeoCode(null);
		}

		ProductProcessor processor = (ProductProcessor) baseForm.getBusinessObjectFactory().createObject(
				"productProcessorImplementer");
		Map results = processor.addProduct(PropertyUtils.describe(form));

		// Successful insert
		if (results == null || results.get("resultCode") == null || results.get("resultCode").equals("0")) {

			// Set the success messages
			baseForm.getMessages().add(ActionMessages.GLOBAL_MESSAGE, new ActionMessage("messages.recordAdded"));
			if (results != null && results.get("newRecord") != null) {
				Product newRecord = (Product) results.get("newRecord");

				// Set a request scope variable for sub classes to use
				request.setAttribute("newProductID", new Integer(newRecord.getProductID()));

				// Set success message
				baseForm.getMessages().add(
						ActionMessages.GLOBAL_MESSAGE,
						new ActionMessage("messages.editNewRecord", "ProductEditForm.do?productID="
								+ newRecord.getProductID()));

				// Optionally update the Lucene index
				updateAdvancedSearchIndex(baseForm, new Integer(newRecord.getProductID()));
			}

			// If successful, signal the form to be reinstantiated.
			request.setAttribute("reset", "true");
		} else if (results != null && results.get("existingRecord") != null) {

			// Set the error messages
			Product existingRecord = (Product) results.get("existingRecord");
			String duplicateField = (String) results.get("duplicateField");
			String existingCode = existingRecord.getCode();
			if (duplicateField.equals("seoCode")) {
				existingCode = existingRecord.getSeoCode();
			}
			baseForm.getErrors().add(ActionMessages.GLOBAL_MESSAGE,
					new ActionMessage("errors.resultCode" + results.get("resultCode"), duplicateField, existingCode));
			baseForm.getErrors().add(
					ActionMessages.GLOBAL_MESSAGE,
					new ActionMessage("errors.editExistingRecord", "ProductEditForm.do?productID="
							+ existingRecord.getProductID(), duplicateField, existingCode));
		}
		saveMessages(baseForm.getRequest(), baseForm.getMessages());
		saveErrors(baseForm.getRequest(), (ActionMessages) baseForm.getErrors());
		return mapping.findForward("success");
	}
}
