package com.softslate.commerce.administrator.product;

import java.util.Arrays;
import java.util.Collection;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.beanutils.PropertyUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;

import com.softslate.commerce.businessobjects.product.Category;
import com.softslate.commerce.businessobjects.product.CategoryProcessor;
import com.softslate.commerce.businessobjects.product.ManufacturerProcessor;
import com.softslate.commerce.customer.core.BaseAction;

/**
 * Struts action class corresponding to requests to
 * <code>/administrator/ProductAddForm.do</code>. Creates an empty
 * <code>ProductAddEditForm</code> object.
 * 
 * @author David Tobey
 * 
 */
public class ProductAddFormAction extends BaseAction {

	static Log log = LogFactory.getLog(ProductAddFormAction.class);

	public ActionForward execute(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response)
			throws Exception {
		if (log.isDebugEnabled())
			log.debug("Starting action.");

		ProductAddEditForm baseForm = (ProductAddEditForm) form;

		if (request.getAttribute("reset") != null
				&& request.getAttribute("reset").equals("true")) {
			Class classDefinition = baseForm.getClass();
			baseForm = (ProductAddEditForm) classDefinition.newInstance();
		}

		baseForm.initializeProperties(mapping, request);
		request.setAttribute("productAddEditForm", baseForm);

		baseForm.setCategoryID(request.getParameter("categoryID"));

		// Load the parent if an id is present
		Category category = null;
		if (baseForm.getCategoryID() != null
				&& baseForm.getCategoryID().length() > 0) {
			CategoryProcessor categoryProcessor = (CategoryProcessor) baseForm
					.getBusinessObjectFactory().createObject(
							"categoryProcessorImplementer");
			category = categoryProcessor.loadCategoryFromID(PropertyUtils
					.describe(baseForm));
			request.setAttribute("category", category);
		}

		// Load all categories for the primary category drop down.
		CategoryProcessor catProcessor = (CategoryProcessor) baseForm
				.getBusinessObjectFactory().createObject(
						"categoryProcessorImplementer");
		Collection categories = catProcessor.loadAllCategories();
		baseForm.getRequest().setAttribute("categories", categories);

		// Load all manufacturers for the manufacturer drop down.
		ManufacturerProcessor manfProcessor = (ManufacturerProcessor) baseForm
				.getBusinessObjectFactory().createObject(
						"manufacturerProcessorImplementer");
		Collection manufacturers = manfProcessor.loadAllManufacturers();
		request.setAttribute("manufacturerCollection", manufacturers);

		// Navigation
		// Create breadcrumbing collection
		Collection bcs = Arrays.asList(new String[] { "Home", "Products",
				"Add Product" });
		request.setAttribute("breadcrumbs", bcs);

		return mapping.findForward("success");
	}
}
