package com.softslate.commerce.administrator.shipping;

import java.util.Arrays;
import java.util.Collection;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.beanutils.BeanUtils;
import org.apache.commons.beanutils.PropertyUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;

import com.softslate.commerce.businessobjects.shipping.ShippingMethod;
import com.softslate.commerce.businessobjects.shipping.ShippingMethodProcessor;
import com.softslate.commerce.customer.core.BaseAction;

/**
 * Struts action class corresponding to requests to
 * "/administrator/ShippingMethodEditForm.do". Loads in the current values of
 * the record being edited and populates a
 * <code>ShippingMethodAddEditForm</code> object.
 * 
 * @author David Tobey
 * 
 */
public class ShippingMethodEditFormAction extends BaseAction {

	static Log log = LogFactory.getLog(ShippingMethodAddFormAction.class);

	public ActionForward execute(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response)
			throws Exception {
		if (log.isDebugEnabled())
			log.debug("Starting action.");
		ShippingMethodAddEditForm baseForm = (ShippingMethodAddEditForm) form;
		baseForm.initializeProperties(mapping, request);

		// Get the record to edit
		ShippingMethodProcessor processor = (ShippingMethodProcessor) baseForm
				.getBusinessObjectFactory().createObject(
						"shippingMethodProcessorImplementer");
		ShippingMethod shippingMethod = processor
				.loadShippingMethodFromID(PropertyUtils.describe(form));
		BeanUtils.copyProperties(baseForm, shippingMethod);

		// Enforce role permissions
		if (!request.isUserInRole("superuser")
				&& !request.isUserInRole("administrator")
				&& !request.isUserInRole("shippingEdit")) {
			request.setAttribute("editPermission", "false");
		} else {
			request.setAttribute("editPermission", "true");
		}

		// Create breadcrumbing collection
		Collection bcs = Arrays.asList(new String[] { "Home",
				"Shipping Methods", "Shipping Method Details" });
		request.setAttribute("breadcrumbs", bcs);

		request.setAttribute("shippingMethod", shippingMethod);
		request.setAttribute("shippingMethodAddEditForm", baseForm);
		return mapping.findForward("success");
	}
}
