package com.softslate.commerce.businessobjects.core;

import org.hibernate.envers.AuditTable;
import org.hibernate.envers.Audited;

/**
 * Interface representing a single application setting stored in the
 * <code>sscSetting</code> database table.
 * <p>
 * When the application needs to create an instance that implements
 * <code>Setting</code>,
 * {@link com.softslate.commerce.businessobjects.core.BusinessObjectFactory}
 * finds the name of the Java class to instantiate from the "settingImplementer"
 * setting in the <code>appComponents.properties</code> file.
 * <p>
 * The default "settingImplementer" is
 * {@link com.softslate.commerce.businessobjects.core.SettingBean}.
 * 
 * @author David Tobey
 */
@Audited
@AuditTable("AudSetting")
public interface Setting extends BusinessObject {
    abstract int getSettingID();

    abstract void setSettingID(int settingID);

    abstract String getCode();

    abstract void setCode(String code);

    abstract String getName();

    abstract void setName(String name);

    abstract String getDescription();

    abstract void setDescription(String description);

    abstract String getType();

    abstract void setType(String type);

    abstract String getValueType();

    abstract void setValueType(String valueType);

    abstract Integer getIntegerValue();

    abstract void setIntegerValue(Integer integerValue);

    abstract String getSmallValue();

    abstract void setSmallValue(String smallValue);

    abstract String getMediumValue();

    abstract void setMediumValue(String mediumValue);

    abstract String getTextValue();

    abstract void setTextValue(String textValue);
}
