package com.softslate.commerce.businessobjects.customer;

import java.io.Serializable;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.hibernate.envers.AuditTable;
import org.hibernate.envers.Audited;

import com.softslate.commerce.businessobjects.core.BaseBusinessObject;
import com.softslate.commerce.businessobjects.product.Product;

/**
 * Object representing a product under a wish list in the system.
 * <code>CustomerWishListProductBean</code> is the default implementation of the
 * <code>CustomerWishListProduct</code> interface for the application.
 * <p>
 * The database stores wish list products in the
 * <code>sscCustomerWishListProduct</code> table, and this interface can be used to
 * represent a single row of that table. The Hibernate configuration file
 * <code>CustomerWishListProductBean.hbm.xml</code> maps that table's columns to the
 * properties in this object.
 * 
 * @author Jason McWilliams
 */
@Audited
@AuditTable("AudCustomerWishListProduct")
public class CustomerWishListProductBean extends BaseBusinessObject implements
		CustomerWishListProduct, Serializable {

	private static final long serialVersionUID = 1L;

	static Log log = LogFactory.getLog(CustomerWishListProductBean.class);

	private int customerWishListProductID = 0;

	public int getCustomerWishListProductID() {
		return (this.customerWishListProductID);
	}

	public void setCustomerWishListProductID(int customerWishListProductID) {
		this.customerWishListProductID = customerWishListProductID;
	}

	private int productID = 0;

	public int getProductID() {
		return (this.productID);
	}

	public void setProductID(int productID) {
		this.productID = productID;
	}

	private int customerWishListID = 0;

	public int getCustomerWishListID() {
		return (this.customerWishListID);
	}

	public void setCustomerWishListID(int customerWishListID) {
		this.customerWishListID = customerWishListID;
	}

	private Product product;

	public Product getProduct() {
		return product;
	}

	public void setProduct(Product product) {
		this.product = product;
	}

	private CustomerWishList customerWishList;

	public CustomerWishList getCustomerWishList() {
		return customerWishList;
	}

	public void setCustomerWishList(CustomerWishList customerWishList) {
		this.customerWishList = customerWishList;
	}
}
