package com.softslate.commerce.businessobjects.payment;

import java.util.Collection;
import java.util.Map;

import com.softslate.commerce.businessobjects.core.BusinessProcessor;

/**
 * Interface for business logic methods having to do with payments. Created in
 * the Struts layer in response to requests for viewing, editing, adding, and
 * deleting payments.
 * <p>
 * When the application needs to create an instance that implements
 * <code>PaymentProcessor</code>,
 * {@link com.softslate.commerce.businessobjects.core.BusinessObjectFactory}<!--
 * --> finds the name of the Java class to instantiate from the
 * "paymentProcessorImplementer" setting in the
 * <code>appComponents.properties</code> file.
 * <p>
 * The default "paymentProcessorImplementer" is
 * {@link com.softslate.commerce.businessobjects.payment.PaymentProcessor}.
 * 
 * @author David Tobey
 * 
 */
public interface AdminPaymentProcessor extends BusinessProcessor {

	abstract Map loadPaymentsAndCount(Map parameters) throws Exception;

	abstract Map addPayment(Map parameters) throws Exception;

	abstract Payment loadPaymentFromID(Map parameters) throws Exception;

	abstract Map editPayment(Map parameters) throws Exception;

	abstract Map editDeletePayments(Map parameters) throws Exception;

	abstract void deletePayment(Map parameters) throws Exception;

	abstract Collection loadCardInfoStatistics() throws Exception;

	abstract int purgeCardInfo(Map parameters) throws Exception;
}
