package com.softslate.commerce.businessobjects.product;

import java.util.Collection;
import java.util.Map;

import com.softslate.commerce.businessobjects.core.BusinessProcessor;

/**
 * Interface for business logic methods having to do with attributes. Created in
 * the Struts layer in response to requests for viewing, editing, adding, and
 * deleting attributes.
 * <p>
 * When the application needs to create an instance that implements
 * <code>AttributeProcessor</code>,
 * {@link com.softslate.commerce.businessobjects.core.BusinessObjectFactory}
 * finds the name of the Java class to instantiate from the
 * "attributeProcessorImplementer" setting in the
 * <code>appComponents.properties</code> file.
 * <p>
 * The default "attributeProcessorImplementer" is
 * {@link com.softslate.commerce.businessobjects.product.AttributeProcessor}.
 * 
 * @author David Tobey
 * 
 */
public interface AttributeProcessor extends BusinessProcessor {
	abstract Collection loadAllAttributes() throws Exception;

	abstract Map loadAttributesAndCount(Map parameters) throws Exception;

	abstract Map addAttribute(Map parameters) throws Exception;

	abstract Attribute loadAttributeFromID(Map parameters) throws Exception;

	abstract Map editAttribute(Map parameters) throws Exception;

	abstract Map editDeleteAttributes(Map parameters) throws Exception;

	abstract void deleteAttribute(Map parameters) throws Exception;

	abstract int loadMaxAttributeID() throws Exception;
}
