package com.softslate.commerce.businessobjects.product;

import java.util.Map;

import org.apache.commons.beanutils.BeanUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.softslate.commerce.businessobjects.core.BaseBusinessProcessor;
import com.softslate.commerce.daos.product.CategorySettingDAO;

/**
 * Processes business logic having to do with category settings. Created in the
 * Struts layer in response to requests for viewing, editing, adding, and
 * deleting category settings.
 * <p>
 * <code>BasicCategorySettingProcessor</code> is the default implementation of
 * the <code>CategorySettingProcessor</code> interface for the application.
 * 
 * @author David Tobey
 */
public class BasicCategorySettingProcessor extends BaseBusinessProcessor
        implements CategorySettingProcessor {

    static Log log = LogFactory.getLog(BasicCategorySettingProcessor.class);

    public void addCategorySetting(Map parameters) throws Exception {
        if (log.isInfoEnabled() && getUser().getAdministrator() != null) {
            String code = (String) parameters.get("code");
            log.info(getUser().getAdministrator().getUserName()
                    + ": Adding category setting: " + code);
        }
        CategorySetting categorySetting =  getBusinessObjectFactory()
                .createObject(CategorySetting.class);
        BeanUtils.copyProperties(categorySetting, parameters);
        Category category = (Category) getBusinessObjectFactory().createObject(
                "categoryImplementer");
        category.setCategoryID(categorySetting.getCategoryID());
        categorySetting.setCategory(category);
        CategorySettingDAO dao =  getDaoFactory()
                .createDAO(CategorySettingDAO.class);
        dao.setCategorySetting(categorySetting);
        dao.insertCategorySetting(true);
        return;
    }

    public CategorySetting loadCategorySettingFromID(Map parameters)
            throws Exception {
        if (log.isInfoEnabled() && getUser().getAdministrator() != null)
            log.info(getUser().getAdministrator().getUserName()
                    + ": Loading category setting from id");
        CategorySetting categorySetting =  getBusinessObjectFactory()
                .createObject(CategorySetting.class);
        CategorySettingDAO dao =  getDaoFactory()
                .createDAO(CategorySettingDAO.class);
        BeanUtils.copyProperties(categorySetting, parameters);
        dao.setCategorySetting(categorySetting);
        dao.loadCategorySettingFromID();
        return categorySetting;
    }

    public void editCategorySetting(Map parameters) throws Exception {
        if (log.isInfoEnabled() && getUser().getAdministrator() != null) {
            String code = (String) parameters.get("code");
            log.info(getUser().getAdministrator().getUserName()
                    + ": Editing category setting: " + code);
        }
        CategorySetting categorySetting =  getBusinessObjectFactory()
                .createObject(CategorySetting.class);
        BeanUtils.copyProperties(categorySetting, parameters);
        Category category = (Category) getBusinessObjectFactory().createObject(
                "categoryImplementer");
        category.setCategoryID(categorySetting.getCategoryID());
        categorySetting.setCategory(category);
        CategorySettingDAO dao =  getDaoFactory()
                .createDAO(CategorySettingDAO.class);
        dao.setCategorySetting(categorySetting);
        dao.updateCategorySetting(true);
        return;
    }

    public void deleteCategorySetting(Map parameters) throws Exception {
        if (log.isInfoEnabled() && getUser().getAdministrator() != null) {
            String code = (String) parameters.get("code");
            log.info(getUser().getAdministrator().getUserName()
                    + ": Deleting category setting: " + code);
        }
        CategorySetting categorySetting =  getBusinessObjectFactory()
                .createObject(CategorySetting.class);
        BeanUtils.copyProperties(categorySetting, parameters);
        CategorySettingDAO dao =  getDaoFactory()
                .createDAO(CategorySettingDAO.class);
        dao.setCategorySetting(categorySetting);
        dao.deleteCategorySetting(true);
        return;
    }
}
