package com.softslate.commerce.businessobjects.product;

import org.hibernate.envers.AuditTable;
import org.hibernate.envers.Audited;

import com.softslate.commerce.businessobjects.core.BusinessObject;

/**
 * Interface representing a category setting within the system.
 * <p>
 * When the application needs to create an instance that implements
 * <code>sscCategorySetting</code>,
 * {@link com.softslate.commerce.businessobjects.core.BusinessObjectFactory}
 * finds the name of the Java class to instantiate from the
 * "categorySettingImplementer" setting in the
 * <code>appComponents.properties</code> file.
 * <p>
 * The default "categorySettingImplementer" is
 * {@link com.softslate.commerce.businessobjects.product.CategorySettingBean}.
 * 
 * @author David Tobey
 */
@Audited
@AuditTable("AudCategorySetting")
public interface CategorySetting extends BusinessObject {
	abstract int getCategorySettingID();

	abstract void setCategorySettingID(int categorySettingID);

	abstract Category getCategory();

	abstract void setCategory(Category category);

	abstract String getCode();

	abstract void setCode(String code);

	abstract String getDescription();

	abstract void setDescription(String description);

	abstract int getIntegerValue();

	abstract void setIntegerValue(int integerValue);

	abstract String getMediumValue();

	abstract void setMediumValue(String mediumValue);

	abstract String getName();

	abstract void setName(String name);

	abstract int getCategoryID();

	abstract void setCategoryID(int categoryID);

	abstract String getSmallValue();

	abstract void setSmallValue(String smallValue);

	abstract String getTextValue();

	abstract void setTextValue(String textValue);

	abstract String getType();

	abstract void setType(String type);

	abstract String getValueType();

	abstract void setValueType(String valueType);

	abstract String getValue();
}
