package com.softslate.commerce.businessobjects.product;

import java.util.Collection;
import java.util.Map;

import org.hibernate.envers.AuditTable;
import org.hibernate.envers.Audited;

import com.softslate.commerce.businessobjects.core.BusinessObject;
import com.softslate.commerce.businessobjects.order.OrderItem;

/**
 * Interface representing a discount within the system.
 * <p>
 * When the application needs to create an instance that implements
 * <code>sscDiscount</code>,
 * {@link com.softslate.commerce.businessobjects.core.BusinessObjectFactory}
 * finds the name of the Java class to instantiate from the
 * "discountImplementer" setting in the <code>appComponents.properties</code>
 * file.
 * <p>
 * The default "discountImplementer" is
 * {@link com.softslate.commerce.businessobjects.product.DiscountBean}.
 * 
 * @author David Tobey
 */
@Audited
@AuditTable("AudDiscount")
public interface Discount extends BusinessObject {
	abstract String getCouponCode();

	abstract void setCouponCode(String couponCode);

	abstract boolean getIsTimesPerCustomer();

	abstract void setIsTimesPerCustomer(boolean isTimesPerCustomer);

	abstract String getItemsAppliesType();

	abstract void setItemsAppliesType(String itemsAppliesType);

	abstract String getItemsAppliesValue();

	abstract void setItemsAppliesValue(String itemsAppliesValue);

	abstract String getUsersAppliesType();

	abstract void setUsersAppliesType(String usersAppliesType);

	abstract String getUsersAppliesValue();

	abstract void setUsersAppliesValue(String usersAppliesValue);

	abstract String getCartItemsType();

	abstract void setCartItemsType(String cartItemsType);

	abstract String getCartItemsValue();

	abstract void setCartItemsValue(String cartItemsValue);
	
	abstract Double getReward();

	abstract void setReward(Double reward);

	abstract String getRewardType();

	abstract void setRewardType(String rewardType);

	abstract void setActive(boolean isActive);

	abstract String getCode();

	abstract void setCode(String code);

	abstract String getDescription();

	abstract void setDescription(String description);

	abstract int getDiscountID();

	abstract void setDiscountID(int discountID);

	abstract int getDiscountOrder();

	abstract void setDiscountOrder(int discountOrder);

	abstract String getExpirationDate();

	abstract void setExpirationDate(String expirationDate);

	abstract String getExtra1();

	abstract void setExtra1(String extra1);

	abstract String getExtra2();

	abstract void setExtra2(String extra2);

	abstract String getExtra3();

	abstract void setExtra3(String extra3);

	abstract boolean getIsActive();

	abstract void setIsActive(boolean isActive);

	abstract Integer getMaxTimesUsed();

	abstract void setMaxTimesUsed(Integer maxTimesUsed);

	abstract String getName();

	abstract void setName(String name);

	abstract String getStartDate();

	abstract void setStartDate(String startDate);

	abstract Integer getTimesUsed();

	abstract void setTimesUsed(Integer timesUsed);
	
	abstract Integer getOrderItemID();

	abstract void setOrderItemID(Integer orderItemID);
	
	abstract OrderItem getOrderItem();

	abstract void setOrderItem(OrderItem orderItem);

	abstract Collection getSkuDiscounts();

	abstract void setSkuDiscounts(Collection skuDiscounts);

	abstract Collection getDiscountRanges();

	abstract void setDiscountRanges(Collection discountRanges);

	abstract Collection getOrderItems();

	abstract void setOrderItems(Collection orderItems);

	abstract Integer getTotalQuantity();

	abstract void setTotalQuantity(Integer totalQuantity);

	abstract Double getTotalAmount();

	abstract void setTotalAmount(Double totalAmount);

	abstract Double getTotalWeight();

	abstract void setTotalWeight(Double totalWeight);
	
	abstract Map getDiscountInformation();

	abstract void setDiscountInformation(Map discountInformation);

}
