package com.softslate.commerce.businessobjects.product;

import java.util.Collection;
import java.util.Map;

import com.softslate.commerce.businessobjects.core.BusinessProcessor;

/**
 * Interface for business logic methods having to do with discounts. Created in
 * the Struts layer in response to requests for viewing, editing, adding, and
 * deleting discounts.
 * <p>
 * When the application needs to create an instance that implements
 * <code>DiscountProcessor</code>,
 * {@link com.softslate.commerce.businessobjects.core.BusinessObjectFactory}
 * finds the name of the Java class to instantiate from the
 * "discountProcessorImplementer" setting in the
 * <code>appComponents.properties</code> file.
 * <p>
 * The default "discountProcessorImplementer" is
 * {@link com.softslate.commerce.businessobjects.product.DiscountProcessor}.
 * 
 * @author David Tobey
 */
public interface DiscountProcessor extends BusinessProcessor {

	abstract Collection loadStoreDiscounts() throws Exception;

	abstract Map loadDiscountsAndCount(Map parameters) throws Exception;

	abstract Map addDiscount(Map parameters) throws Exception;

	abstract Discount loadDiscountFromID(Map parameters) throws Exception;

	abstract Map editDiscount(Map parameters) throws Exception;

	abstract Map editDeleteDiscounts(Map parameters) throws Exception;

	abstract void deleteDiscount(Map parameters) throws Exception;

	abstract Collection loadAllCoupons() throws Exception;
}
