package com.softslate.commerce.businessobjects.product;

import java.util.Collection;
import java.util.Locale;
import java.util.Map;

import org.hibernate.envers.AuditTable;
import org.hibernate.envers.Audited;

import com.softslate.commerce.businessobjects.core.BusinessObject;

/**
 * Interface representing a product within the system. A product is the
 * purchaseable unit of the store.
 * <p>
 * When the application needs to create an instance that implements
 * <code>Product</code>,
 * {@link com.softslate.commerce.businessobjects.core.BusinessObjectFactory}
 * finds the name of the Java class to instantiate from the "productImplementer"
 * setting in the <code>appComponents.properties</code> file.
 * <p>
 * The default "productImplementer" is
 * {@link com.softslate.commerce.businessobjects.product.ProductBean}.
 * 
 * @author David Tobey
 * 
 */
@Audited
@AuditTable("AudProduct")
public interface Product extends BusinessObject {
	abstract int getProductID();

	abstract void setProductID(int productID);

	abstract String getCode();

	abstract void setCode(String code);

	abstract String getSeoCode();

	abstract void setSeoCode(String seoCode);

	abstract String getName();

	abstract void setName(String name);

	abstract boolean getIsActive();

	abstract void setIsActive(boolean isActive);

	abstract String getKeywords();

	abstract void setKeywords(String keywords);

	abstract String getShortDescription();

	abstract void setShortDescription(String shortDescription);

	abstract String getDescription();

	abstract void setDescription(String description);

	abstract Double getUnitCost();

	abstract void setUnitCost(Double unitCost);

	abstract Double getUnitPrice();

	abstract void setUnitPrice(Double unitPrice);

	abstract Double getAltPrice();

	abstract void setAltPrice(Double altPrice);

	abstract Double getWeight();

	abstract void setWeight(Double weight);

	abstract String getHeader();

	abstract void setHeader(String header);

	abstract String getFooter();

	abstract void setFooter(String footer);

	abstract boolean getIsTaxed();

	abstract void setIsTaxed(boolean isTaxed);

	abstract String getFormattedCost();

	abstract void setFormattedCost(String formattedCost);

	abstract String getFormattedPrice();

	abstract void setFormattedPrice(String formattedPrice);

	abstract String getFormattedAltPrice();

	abstract void setFormattedAltPrice(String formattedAltPrice);

	abstract String getSmallImage();

	abstract void setSmallImage(String smallImage);

	abstract String getMediumImage();

	abstract void setMediumImage(String mediumImage);

	abstract String getLargeImage();

	abstract void setLargeImage(String largeImage);

	abstract String getExtra1();

	abstract void setExtra1(String extra1);

	abstract String getExtra2();

	abstract void setExtra2(String extra2);

	abstract String getExtra3();

	abstract void setExtra3(String extra3);

	abstract String getExtra4();

	abstract void setExtra4(String extra4);

	abstract String getExtra5();

	abstract void setExtra5(String extra5);

	abstract String getCreated();

	abstract void setCreated(String created);

	abstract Integer getPrimaryCategoryID();

	abstract void setPrimaryCategoryID(Integer primaryCategoryID);

	abstract int getProductOrder();

	abstract void setProductOrder(int productOrder);

	abstract Collection getAttributesAndOptions();

	abstract void setAttributesAndOptions(Collection attributesAndOptions);

	abstract void addFormatting(Locale locale, String currencyCode);

	abstract Map getProductSettings();

	abstract void setProductSettings(Map productSettings);

	public Collection getProductCategories();

	public void setProductCategories(Collection productCategories);

	public Category getPrimaryCategory();

	public void setPrimaryCategory(Category primaryCategory);

	public Collection getProductAttributes();

	public void setProductAttributes(Collection productAttributes);

	public Collection getSkus();

	public void setSkus(Collection skus);

	public Map getSkuMap();

	public Collection getRelatedProducts();

	public void setRelatedProducts(Collection relatedProducts);

	public Collection getRelatedProductsAdmin();

	public void setRelatedProductsAdmin(Collection relatedProductsAdmin);

	abstract Integer getManufacturerID();

	abstract void setManufacturerID(Integer manufacturerID);

	abstract Manufacturer getManufacturer();

	abstract void setManufacturer(Manufacturer manufacturer);
	
	public Collection getCustomerWishListProducts();

	public void setCustomerWishListProducts(Collection customerWishListProducts);

	public Collection getProductReviews();

	public void setProductReviews(Collection productReviews);
}
