package com.softslate.commerce.customer.core;

import java.io.IOException;
import java.io.UnsupportedEncodingException;

import javax.servlet.Filter;
import javax.servlet.FilterChain;
import javax.servlet.FilterConfig;
import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.http.HttpServletRequest;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.softslate.commerce.businessobjects.core.Settings;

/**
 * 
 * @author David Tobey
 * 
 */
public class CharacterEncodingFilter extends BaseFilter implements Filter {
	static Log log = LogFactory.getLog(CharacterEncodingFilter.class);
	private FilterConfig filterConfig = null;

	public void init(FilterConfig filterConfig) throws ServletException {
		this.filterConfig = filterConfig;
	}

	public void destroy() {
	}

	public void doFilter(ServletRequest request, ServletResponse response,
			FilterChain chain) throws IOException, ServletException {

		// Must cast incoming request to HttpServletRequest
		HttpServletRequest req = (HttpServletRequest) request;

		// Ignore the request if it is an image.
		if (isStaticRequest(req)) {
			chain.doFilter(request, response);
			return;
		}

		// Try grabbing the settings from the application scope
		Settings settings = (Settings) filterConfig.getServletContext()
				.getAttribute("settings");
		if (settings != null) {

			// Set the character encoding based on the setting
			String charEncoding = (String) settings
					.getValue("characterEncoding");
			if (charEncoding != null && charEncoding.length() > 0) {
				if (log.isInfoEnabled())
					log.info("Setting encoding to: " + charEncoding);
				try {
					request.setCharacterEncoding(charEncoding);
					response.setCharacterEncoding(charEncoding);
				} catch (UnsupportedEncodingException uee) {
					if (log.isErrorEnabled())
						log.error("Error setting character encoding: ", uee);
				}
			}
		}
		
		// Pass the request along
		chain.doFilter(request, response);
		return;
	}
}
