package com.softslate.commerce.customer.customer;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.action.ActionMessage;
import org.apache.struts.action.ActionMessages;

import com.softslate.commerce.businessobjects.customer.CustomerProcessor;
import com.softslate.commerce.customer.core.BaseAction;
import com.softslate.commerce.customer.core.BaseForm;

/**
 * Struts action class used for requests to view a customer's order history.
 * Used for requests to "/AccountHistory.do".
 * <p>
 * The <code>execute</code> method of this class invokes
 * {@link com.softslate.commerce.businessobjects.customer.CustomerProcessor#loadOrderHistory()}
 * to load the customer's history of orders from the database. It places a
 * <code>Collection</code> of
 * {@link com.softslate.commerce.businessobjects.order.Order} objects in the
 * request scope in an attribute named "historyList".
 * 
 * @author David Tobey
 * 
 */
public class AccountHistoryAction extends BaseAction {
	static Log log = LogFactory.getLog(AccountHistoryAction.class);

	public ActionForward execute(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response)
			throws Exception {
		if (log.isInfoEnabled())
			log.info("Starting action.");
		BaseForm baseForm = new BaseForm();
		baseForm.setServlet(getServlet());
		baseForm.initializeProperties(mapping, request);
		synchronized (baseForm.getRequest().getSession()) {

			// If the user is not logged in, it's an error.
			if (baseForm.getUser().getCustomer() == null) {

				// Preserve the original request so the customer is redirected
				// back here after logging in
				request
						.setAttribute("originalRequest", request
								.getRequestURI());

				// Record the error
				baseForm.getErrors().add(ActionMessages.GLOBAL_MESSAGE,
						new ActionMessage("errors.userNotLoggedIn"));
				saveErrors(baseForm.getRequest(), (ActionMessages) baseForm
						.getErrors());
				if (log.isInfoEnabled() && !baseForm.getErrors().isEmpty())
					log.info("Validation error(s): "
							+ baseForm.getErrors().toString());
				return mapping.findForward("notLoggedIn");
			}

			// Query for the customer's order history
			CustomerProcessor customerProcessor = (CustomerProcessor) baseForm
					.getBusinessObjectFactory().createObject(
							"customerProcessorImplementer");
			customerProcessor.loadOrderHistory();

			// Save the orders to the request scope
			baseForm.getRequest().setAttribute("historyList",
					baseForm.getUser().getCustomer().getOrderHistory());

			// Define the starting index of the list
			String startingItem = "0";
			if (request.getParameter("startingItem") != null) {
				startingItem = request.getParameter("startingItem");
			}
			baseForm.getRequest().setAttribute("startingItem", startingItem);

			return mapping.findForward("success");
		}
	}
}
