package com.softslate.commerce.customer.customer;

import java.util.Properties;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.lang.StringUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.action.ActionMessage;
import org.apache.struts.action.ActionMessages;

import com.softslate.commerce.businessobjects.customer.Customer;
import com.softslate.commerce.businessobjects.order.Order;
import com.softslate.commerce.businessobjects.payment.authorizeNetCIM.AuthorizeNetCIMPaymentProfileMin;
import com.softslate.commerce.businessobjects.payment.authorizeNetCIM.AuthorizeNetCIMProcessor;
import com.softslate.commerce.customer.order.CIMBillingAddressForm;
import com.softslate.commerce.customer.core.BaseAction;

/**
 * 
 * @author Jason McWilliams
 * 
 */
public class EditCIMProfileFormAction extends BaseAction {
	static Log log = LogFactory.getLog(EditCIMProfileFormAction.class);

	public ActionForward execute(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response)
			throws Exception {
		if (log.isInfoEnabled())
			log.info("Starting action.");

		CIMBillingAddressForm baseForm = (CIMBillingAddressForm) form;
		baseForm.setServlet(getServlet());
		baseForm.initializeProperties(mapping, request);

		synchronized (request.getSession()) {
			Properties appComponents = (Properties) getServlet().getServletContext().getAttribute("appComponents");
			String activePaymentProcessors = appComponents.getProperty("activePaymentProcessors");
			if (!activePaymentProcessors.contains("AuthorizeNetCIMProcessor")) {
				if (log.isInfoEnabled())
					log.info("CIM is not an active payment processor.");
				baseForm.getErrors().add(ActionMessages.GLOBAL_MESSAGE,
						new ActionMessage("errors.cimManageProfilesNotAllowed"));
				saveErrors(baseForm.getRequest(), (ActionMessages) baseForm.getErrors());
				return mapping.findForward("failure");
			}

			// Check Settings
			String authorizeNetCIMHostedMethod = (String) baseForm.getSettings()
					.getValue("authorizeNetCIMHostedMethod");
			if (!"API Only".equals(authorizeNetCIMHostedMethod)) {
				if (log.isInfoEnabled())
					log.info("API Only is not turned on");
				baseForm.getErrors().add(ActionMessages.GLOBAL_MESSAGE,
						new ActionMessage("errors.cimManageProfilesNotAllowedAPI"));
				saveErrors(baseForm.getRequest(), (ActionMessages) baseForm.getErrors());
				return mapping.findForward("failure");
			}

			Customer customer = baseForm.getUser().getCustomer();
			Order order = baseForm.getUser().getOrder();
			AuthorizeNetCIMProcessor cimProcessor =  baseForm.getBusinessObjectFactory()
					.createObject(AuthorizeNetCIMProcessor.class);
			cimProcessor.initialize();
			AuthorizeNetCIMPaymentProfileMin paymentProfile = cimProcessor.findMatchingPaymentProfile(customer, order,
					baseForm.getCimPaymentProfileID());
			if (paymentProfile != null) {
				cimProcessor.copyPaymentPaymentProfileToAddressForm(paymentProfile, baseForm);
				// email is not sent with payments, only profiles, try to find
				// an email to use in case it is required in the form
				if (customer != null && StringUtils.isNotBlank(customer.getEmail1())) {
					baseForm.setBillingEmail1(customer.getEmail1());
				} else if (order != null && StringUtils.isNotBlank(order.getEmail1())) {
					baseForm.setBillingEmail1(order.getEmail1());
				}
				baseForm.getRequest().setAttribute("cimPaymentProfile", paymentProfile);
				baseForm.getRequest().setAttribute("addressForm", baseForm);
				return mapping.findForward("success");
			} else {
				baseForm.getErrors().add(ActionMessages.GLOBAL_MESSAGE,
						new ActionMessage("errors.cimManageProfilesGetProfileError"));
				saveErrors(baseForm.getRequest(), (ActionMessages) baseForm.getErrors());
				return mapping.findForward("failure");
			}
		}
	}
}
