package com.softslate.commerce.customer.order;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.action.ActionMessage;
import org.apache.struts.action.ActionMessages;

import com.softslate.commerce.customer.core.BaseForm;

/**
 * Struts action class for submissions to view the checkout shipping form. Used
 * for requests to "/CheckoutShippingForm.do".
 * <p>
 * The <code>execute</code> method of this class invokes
 * {@link com.softslate.commerce.customer.order.CheckoutAction#loadShippingOptions}
 * to prepare the checkout shipping form.
 * 
 * @author David Tobey
 */
public class CheckoutShippingFormAction extends CheckoutAction {
	static Log log = LogFactory.getLog(CheckoutShippingFormAction.class);

	public ActionForward execute(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response)
			throws Exception {
		if (log.isInfoEnabled())
			log.info("Starting action.");

		// Initialize the CheckoutShippingForm
		BaseForm baseForm = (BaseForm) form;
		baseForm.setServlet(getServlet());
		baseForm.initializeProperties(mapping, request);

		// Load the shipping options
		try {
			prepareCheckoutShippingForm(baseForm);
		} catch (Exception ex) {
			if (log.isErrorEnabled())
				log.error("Error loading shipping options: ", ex);
			String message = baseForm.getMessage("order",
					"shipping.processingErrorArg", new String[] { ex
							.getMessage() });
			baseForm.getErrors().add(ActionMessages.GLOBAL_MESSAGE,
					new ActionMessage("errors.detail", message));
			saveErrors(baseForm.getRequest(), (ActionMessages) baseForm
					.getErrors());
			return baseForm.getMapping().findForward(
					"failureLoadShippingOptions");
		}

		return baseForm.getMapping().findForward("success");
	}

}
