package com.softslate.commerce.customer.order;

import javax.servlet.http.HttpServletRequest;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionErrors;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.action.ActionMessage;
import org.apache.struts.action.ActionMessages;

/**
 * Struts form class for submissions to the store's order form. Used for
 * requests to "/OrderProcess.do".
 * <p>
 * <code>OrderForm</code> is a subclass of
 * {@link com.softslate.commerce.customer.order.CheckoutComboShippingForm}, and it
 * inherits its address properties and shipping option property.
 * <p>
 * The <code>validate</code> method of this class runs the
 * <code>validate</code> method of <code>CheckoutComboShippingForm</code> to
 * make sure all the required address fields are present and a shipping option
 * was selected. It then checks to make sure at least one product with a
 * positive quantity was submitted.
 * 
 * @author David Tobey
 */
public class OrderForm extends CheckoutComboShippingForm {

	private static final long serialVersionUID = 3258410621086021938L;

	static Log log = LogFactory.getLog(OrderForm.class);

	private String[] orderProductCodes = {};

	private String[] orderProductQuantities = {};

	public String[] getOrderProductCodes() {
		return orderProductCodes;
	}

	public String[] getOrderProductQuantities() {
		return orderProductQuantities;
	}

	public void setOrderProductCodes(String[] string) {
		orderProductCodes = string;
	}

	public void setOrderProductQuantities(String[] string) {
		orderProductQuantities = string;
	}

	public ActionErrors validate(ActionMapping mapping,
			HttpServletRequest request) {
		if (log.isInfoEnabled())
			log.info("Starting validation.");

		super.validate(mapping, request);

		// Check for catalog mode
		try {
			String catalogMode = (String) getSettings().getValue("catalogMode");
			if (catalogMode.equals("1")) {
				getErrors().add(ActionMessages.GLOBAL_MESSAGE,
						new ActionMessage("errors.catalogMode"));
				return getErrors();
			}
		} catch (Exception e) {
		}

		validateProducts();

		if (log.isDebugEnabled())
			log.debug("Finished validation.");
		if (log.isInfoEnabled() && !getErrors().isEmpty())
			log.info("Validation error(s): " + getErrors().toString());
		return getErrors();
	}

	/**
	 * Ensure that at least one product was submitted with a positive quantity.
	 */
	public void validateProducts() {
		// Check that the user entered a quantity greater than 0 for at least
		// one product.
		if (!formUtils().stringArrayValuesGreaterThan(
				getOrderProductQuantities(), 0)) {
			String message = getMessage("order",
					"orderForm.productQuantityRequired", null);
			getErrors().add(ActionMessages.GLOBAL_MESSAGE,
					new ActionMessage("errors.required", message));
		}
		// Check that at least one product code was submitted.
		if (getOrderProductCodes() == null
				|| getOrderProductCodes().length == 0) {
			String message = getMessage("order", "orderForm.productRequired",
					null);
			getErrors().add(ActionMessages.GLOBAL_MESSAGE,
					new ActionMessage("errors.required", message));
		}
		return;
	}

}
