package com.softslate.commerce.customer.order;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.action.ActionMessage;
import org.apache.struts.action.ActionMessages;

import com.softslate.commerce.businessobjects.product.ProductList;
import com.softslate.commerce.businessobjects.product.ProductListBean;
import com.softslate.commerce.businessobjects.product.ProductProcessor;

/**
 * Struts action class for submissions to view the store's order form. Used for
 * requests to "/OrderForm.do".
 * <p>
 * The <code>execute</code> method of this class populates a
 * <code>ProductList</code> with the products the user can order from the
 * order form. The "orderFormProductCount" setting is used to limit the number
 * of products that are displayed. It then prepares an instance of
 * <code>OrderForm</code> with the addresses and shipping options, and
 * prepares instances of each of the store's "activePaymentForms".
 * 
 * @author David Tobey
 * 
 */
public class OrderFormAction extends CheckoutAction {
	static Log log = LogFactory.getLog(OrderFormAction.class);

	public ActionForward execute(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response)
			throws Exception {
		if (log.isInfoEnabled())
			log.info("Starting action.");
		OrderForm baseForm = (OrderForm) form;
		baseForm.setServlet(getServlet());
		baseForm.initializeProperties(mapping, request);

		// Load in the product list
		ProductList productList = (ProductList) baseForm
				.getBusinessObjectFactory().createObject(
						"productListImplementer");
		String itemsPerPage = (String) baseForm.getSettings().getValue(
				"orderFormProductCount");
		int i = Integer.parseInt(itemsPerPage);
		if (i > 0) {
			ProductProcessor productProcessor = (ProductProcessor) baseForm
					.getBusinessObjectFactory().createObject(
							"productProcessorImplementer");
			productList.setItemsPerPage(Integer.parseInt(itemsPerPage));
			productList.setProductListType(ProductListBean.ALL_ACTIVE);
			productList = productProcessor.getProductList(productList);
			request.setAttribute("productList", productList);

			// Prepare the addresses and shipping.
			baseForm.initializeProperties(mapping, request);
			prepareCheckoutAddressesForm(baseForm);
			prepareCheckoutShippingForm(baseForm);

			// Initialize all the active payment forms.
			preparePaymentForms(baseForm);

			// Copy the order form so the jsp's pick up its properties.
			baseForm.getRequest().setAttribute("addressForm", baseForm);
			baseForm.getRequest().setAttribute("shippingForm", baseForm);
		} else {
			baseForm.getErrors().add(ActionMessages.GLOBAL_MESSAGE,
					new ActionMessage("errors.invalid", "Order Form Product Count setting"));
			saveErrors(baseForm.getRequest(), (ActionMessages) baseForm.getErrors());
			return mapping.findForward("failure");
		}

		return mapping.findForward("success");
	}
}
