package com.softslate.commerce.customer.order;

import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.beanutils.PropertyUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.action.ActionMessage;
import org.apache.struts.action.ActionMessages;

import com.softslate.commerce.businessobjects.order.CartProcessor;

/**
 * Struts action class for submissions to the store's order form. Used for
 * requests to "/OrderProcess.do".
 * <p>
 * The <code>execute</code> method of this class invokes several business
 * methods to create a user's cart, add items to it, and record the submitted
 * address information, shipping information and payment information. In one
 * step, and order is created, shipping information recorded, and payment
 * information processed.
 * 
 * @author David Tobey
 */
public class OrderProcessAction extends CheckoutAction {
	static Log log = LogFactory.getLog(OrderProcessAction.class);

	public ActionForward execute(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response)
			throws Exception {
		if (log.isInfoEnabled())
			log.info("Starting action.");
		OrderForm orderForm = (OrderForm) form;
		synchronized (orderForm.getRequest().getSession()) {
			// Check for a duplicate submission.
			if (orderForm.getRequest().getSession().getAttribute(
					"orderFormSubmitted") != null) {
				orderForm.getErrors().add(ActionMessages.GLOBAL_MESSAGE,
						new ActionMessage("errors.duplicateOrderSubmission"));
				saveErrors(orderForm.getRequest(), (ActionMessages) orderForm
						.getErrors());
				return orderForm.getMapping().findForward(
						"failureDuplicateSubmission");
			}
			// Nullify any existing order in the session.
			orderForm.getUser().setOrder(null);
			// Add items to a new order
			CartProcessor cartProcessor = (CartProcessor) orderForm
					.getBusinessObjectFactory().createObject(
							"cartProcessorImplementer");
			cartProcessor.processAddItems(PropertyUtils.describe(orderForm));
			if (log.isDebugEnabled())
				log.debug("Added items.");
			// Populate the addresses form from the request.
			prepareCheckoutAddressesForm(orderForm);
			// Save the addresses in the new User.
			orderForm.initializeProperties(mapping, request);
			cartProcessor.processCheckoutAddresses(PropertyUtils
					.describe(orderForm));
			if (log.isDebugEnabled())
				log.debug("Processed addresses.");
			// Process inventory and discounting check
			String trackInventory = (String) orderForm.getSettings().getValue(
					"trackInventory");
			if (trackInventory.equals("1")) {
				Map invDiscResults = processInventoryDiscountCheck(orderForm);
				int resultCode = Integer.parseInt((String) invDiscResults
						.get("resultCode"));
				if (resultCode == 0) {
					if (log.isDebugEnabled())
						log.debug("Processed the inventory and discounting.");
				} else if (resultCode == 1) {
					if (log.isDebugEnabled())
						log.debug("Some order items adjusted - must inform");
					return orderForm.getMapping().findForward(
							"failureItemsAdjusted");
				} else if (resultCode == 2) {
					if (log.isDebugEnabled())
						log.debug("All order items removed due to inventory");
					return orderForm.getMapping().findForward(
							"failureAllItemsRemoved");
				}
			}
			// Process the shipping
			String shippingRequired = (String) orderForm.getSettings()
					.getValue("shippingRequired");
			if (shippingRequired.equals("1")) {
				Map shipResults = processShipping(orderForm);
				if (Integer.parseInt((String) shipResults.get("resultCode")) == 0) {
					if (log.isDebugEnabled())
						log.debug("Processed the shipping.");
				} else {
					if (log.isDebugEnabled())
						log.debug("Failed to process shipping.");
					return orderForm.getMapping().findForward("failure");
				}
			}
			// Process the tax
			Map taxResults = processTax(orderForm);
			if (Integer.parseInt((String) taxResults.get("resultCode")) == 0) {
				if (log.isDebugEnabled())
					log.debug("Processed the tax.");
			} else {
				if (log.isDebugEnabled())
					log.debug("Failed to process tax.");
				return orderForm.getMapping().findForward("failure");
			}
			// Set the shipping and tax processed flag.
			orderForm.getRequest().getSession().setAttribute(
					"shippingAndTaxesProcessed", "true");
			// Set the decline login flag (in case the confirm screen is next)
			orderForm.getRequest().getSession().setAttribute(
					"declinedLoginBeforeCheckout", "true");
			// Process the payment
			String paymentRequired = (String) orderForm.getSettings().getValue(
					"paymentRequired");
			if (paymentRequired.equals("1")
					&& (orderForm.getUser().getOrder().getTotal().doubleValue() > 0 || orderForm
							.getUser().getOrder().getSubtotal().doubleValue() > 0)) {
				ActionForward af = processCheckoutPayment(orderForm);
				if (af.getName().equals("success")) {
					// Set a flag to prevent duplicate submissions
					orderForm.getRequest().getSession().setAttribute(
							"orderFormSubmitted", "true");
				}
				return af;
			} else {
				// Mark order complete
				processOrderComplete(orderForm);
				// Set a flag to prevent duplicate submissions
				orderForm.getRequest().getSession().setAttribute(
						"orderFormSubmitted", "true");
				return orderForm.getMapping().findForward("success");
			}
		}
	}
}
