package com.softslate.commerce.customer.payment.cresecure;

import java.util.HashMap;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.lang.StringUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.action.ActionMessage;
import org.apache.struts.action.ActionMessages;

import com.softslate.commerce.businessobjects.order.Order;
import com.softslate.commerce.businessobjects.order.OrderProcessor;
import com.softslate.commerce.customer.order.CheckoutAction;

/**
 * 
 * @author Jason McWilliams
 */

public class CRESecureInvoiceAction extends CheckoutAction {
	static Log log = LogFactory.getLog(CRESecureInvoiceAction.class);

	public ActionForward execute(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response)
			throws Exception {
		if (log.isInfoEnabled())
			log.info("Starting action.");

		CRESecureCompleteForm baseForm = (CRESecureCompleteForm) form;
		baseForm.initializeProperties(mapping, request);

		synchronized (baseForm.getRequest().getSession()) {

			// Pull the completed orderid from their session var to be sure they
			// can view the invoice
			String creorderid = (String) baseForm.getRequest().getSession().getAttribute("creorderid");

			int orderID = 0;
			if (StringUtils.isNotBlank(creorderid)) {
				try {
					orderID = Integer.parseInt(creorderid);
				} catch (Exception ex) {
					orderID = 0;
				}
			}

			if (orderID < 1) {
				if (log.isInfoEnabled())
					log.info("No valid order id in session.");
				baseForm.getErrors().add(ActionMessages.GLOBAL_MESSAGE, new ActionMessage("messages.sessionTimeout"));
				saveErrors(baseForm.getRequest(), (ActionMessages) baseForm.getErrors());
				return baseForm.getMapping().findForward("failure");
			}

			Map params = new HashMap();
			params.put("orderID", Integer.toString(orderID));
			OrderProcessor orderProcessor = (OrderProcessor) baseForm.getBusinessObjectFactory().createObject(
					"orderProcessorImplementer");
			Order order = orderProcessor.loadOrderFromID(params);

			if (order == null || order.getCompleted() == null) {
				if (log.isInfoEnabled())
					log.info("No valid order id in session.");
				baseForm.getErrors().add(ActionMessages.GLOBAL_MESSAGE, new ActionMessage("messages.sessionTimeout"));
				saveErrors(baseForm.getRequest(), (ActionMessages) baseForm.getErrors());
				return baseForm.getMapping().findForward("failure");
			}

			// Set order in baseForm to display on thank you page
			baseForm.getUser().setOrder(order);
			request.setAttribute("user", baseForm.getUser());
			request.setAttribute("baseForm", baseForm);

			// The next request after the invoice should remove the order from
			// the user's session if settings say so. The order could have still
			// been in there to start with, but it does not hurt to pull it back
			// in and reset it in the session
			return mapping.findForward("success");


		}
	}

}
