package com.softslate.commerce.daos.product;

import java.util.Collection;
import java.util.LinkedHashSet;
import java.util.Map;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.hibernate.HibernateException;
import org.hibernate.Query;
import org.hibernate.Session;

import com.softslate.commerce.businessobjects.core.BusinessObjectFactory;
import com.softslate.commerce.businessobjects.product.ProductAttribute;
import com.softslate.commerce.daos.core.BaseAdminGatewayDAOHibernate;
import com.softslate.commerce.daos.core.DataAccessException;

/**
 * Generic DAO that runs miscellaneous queries related to attributes.
 * 
 * @author David Tobey
 * 
 */
public class AttributeGatewayDAOHibernate extends BaseAdminGatewayDAOHibernate
		implements AttributeGatewayDAO {

	static Log log = LogFactory.getLog(AttributeGatewayDAOHibernate.class);

	public Map loadAttributesAndCount(Map parameters) throws Exception {
		return loadItemsAndCount(parameters);
	}

	public Map editDeleteAttributes(Map parameters, BusinessObjectFactory bof)
			throws Exception {
		Map results = editDeleteItems(parameters, bof);
		evictCache();
		return results;
	}

	public Collection loadAllAttributes() throws DataAccessException {
		Collection attributes = null;
		Session session = getDaoFactory().startSession();
		try {
			Query q = getNamedQuery(session, "attribute.selectAll");
			attributes = new LinkedHashSet(q.list());
		} catch (HibernateException he) {
			if (log.isErrorEnabled())
				log.error("Hibernate error: " + he.toString());
			throw new DataAccessException(he);
		}
		return attributes;
	}

	public void insertProductAttribute(ProductAttribute pa)
			throws DataAccessException {
		insertProductAttribute(pa, false);
	}

	public void insertProductAttribute(ProductAttribute pa, boolean commit)
			throws DataAccessException {
		Session session = getDaoFactory().startSession();
		try {
			session.save(pa);
			if (commit) {
				getDaoFactory().commitTransaction();
			}
			evictCache();
		} catch (HibernateException he) {
			if (log.isErrorEnabled())
				log.error("Hibernate error: " + he.toString());
			throw new DataAccessException(he);
		}
		return;
	}

	public int loadMaxAttributeID() throws Exception {
		Session session = getDaoFactory().startSession();
		try {
			Query q = getNamedQuery(session, "product.maxAttributeID");
			Integer maxNum = (Integer) q.uniqueResult();
			if (maxNum == null) {
				return 0;
			}
			return maxNum.intValue();
		} catch (HibernateException he) {
			if (log.isErrorEnabled())
				log.error("Hibernate error: " + he.toString());
			throw new DataAccessException(he);
		}
	}

	public Collection loadAllAttributesInRange(int firstID, int lastID)
			throws Exception {
		if (log.isInfoEnabled())
			log.info("Loading attributes within an id range: " + firstID + "-"
					+ lastID);
		Collection items = null;
		Session session = getDaoFactory().startSession();
		try {
			Query q = getNamedQuery(session, "attribute.selectAllInRange");
			q.setInteger("firstID", firstID);
			q.setInteger("lastID", lastID);
			items = new LinkedHashSet(q.list());
		} catch (HibernateException he) {
			if (log.isErrorEnabled())
				log.error("Hibernate error: " + he.toString());
			throw new DataAccessException(he);
		}
		return items;
	}

}
