package com.softslate.commerce.daos.product;

import org.apache.commons.beanutils.PropertyUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.hibernate.HibernateException;
import org.hibernate.Query;
import org.hibernate.Session;

import com.softslate.commerce.businessobjects.product.Option;
import com.softslate.commerce.daos.core.BaseDAO;
import com.softslate.commerce.daos.core.DAOInterface;
import com.softslate.commerce.daos.core.DataAccessException;

/**
 * Generic DAO to manage the <code>sscOption</code> table of the database.
 * 
 * @author David Tobey
 * 
 */
public class OptionDAOHibernate extends BaseDAO implements DAOInterface,
        OptionDAO {

    static Log log = LogFactory.getLog(OptionDAOHibernate.class);

    private Option option = null;

    public Option getOption() {
        return (this.option);
    }

    public void setOption(Option option) {
        this.option = option;
    }

    public void loadOptionFromID() throws Exception {
        Session session = getDaoFactory().startSession();
        try {
            Option option = (Option) session.get(Option.class, new Integer(
                    getOption().getOptionID()));
            PropertyUtils.copyProperties(getOption(), option);
            setOption(option);
        } catch (HibernateException he) {
            if (log.isErrorEnabled())
                log.error("Hibernate error: " + he.toString());
            throw new DataAccessException(he);
        }
        return;
    }

    public void insertOption() throws DataAccessException {
        insertOption(false);
    }

    public void insertOption(boolean commit) throws DataAccessException {
        Session session = getDaoFactory().startSession();
        try {
            session.save(getOption());
            if (commit) {
                getDaoFactory().commitTransaction();
            }
            evictCache();
        } catch (HibernateException he) {
            if (log.isErrorEnabled())
                log.error("Hibernate error: " + he.toString());
            throw new DataAccessException(he);
        }
        return;
    }

    public void updateOption() throws DataAccessException {
        updateOption(false);
    }

    public void updateOption(boolean commit) throws DataAccessException {
        Session session = getDaoFactory().startSession();
        try {
            session.update(getOption());
            if (commit) {
                getDaoFactory().commitTransaction();
            }
        } catch (HibernateException he) {
            if (log.isErrorEnabled())
                log.error("Hibernate error: " + he.toString());
            throw new DataAccessException(he);
        }
        return;
    }

    public void deleteOption() throws DataAccessException {
        deleteOption(false);
    }

    public void deleteOption(boolean commit) throws DataAccessException {
        Session session = getDaoFactory().startSession();
        try {
            setOption((Option) session.load(Option.class, new Integer(
                    getOption().getOptionID())));
            session.delete(getOption());
            if (commit) {
                getDaoFactory().commitTransaction();
            }
            evictCache();
        } catch (HibernateException he) {
            if (log.isErrorEnabled())
                log.error("Hibernate error: " + he.toString());
            throw new DataAccessException(he);
        }
        return;
    }
    
	
	public void loadOptionFromCode() throws DataAccessException {
		Session session = getDaoFactory().startSession();
		try {
			Query q = getNamedQuery(session, "option.selectOptionFromCodeAndAttributeID");
			q.setString("optionCode", this.getOption().getOptionCode());
			q.setInteger("attributeID", this.getOption().getAttributeID());
			
			Option loadedOption = (Option) q.uniqueResult();
			setOption(loadedOption);
		} catch (HibernateException he) {
			if (log.isErrorEnabled())
				log.error("Hibernate error: " + he.toString());
			throw new DataAccessException(he);
		}
		return;
	}
	
	
	public Object loadObject(Object object) throws Exception 
	{
	  if (null != object && object instanceof Option)
	  {
		  this.setOption((Option)object);
		  if (0 != this.getOption().getOptionID())
		  {			  
			  this.loadOptionFromID();
		  }
		  else if (null != this.getOption().getOptionCode() && 0 != this.getOption().getAttributeID())
		  {
			  this.loadOptionFromCode();
		  }
		  return this.getOption();
	  }
	  return null;
	}

	public boolean deleteObject(Object object, boolean commit) throws Exception 
	{
		  if (null != object && object instanceof Option)
		  {
				  this.setOption((Option)object);
				  this.deleteOption(commit);
				  return true;
		  }
		  return false;
	}

	public Object insertObject(Object object, boolean commit) throws Exception 
	{
		  if (null != object && object instanceof Option)
		  {
			this.setOption((Option)object);
			this.insertOption(commit);
			return this.getOption();
		  }
		  return null;
	}

	public Object updateObject(Object object, boolean commit) throws Exception 
	{
	  if (null != object && object instanceof Option)
	  {
			  this.setOption((Option)object);
			  this.updateOption(commit);
			  return this.getOption();
	  }
	  return null;
	  
	}
    
}
