package com.softslate.commerce.daos.shipping;

import org.apache.commons.beanutils.PropertyUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.hibernate.HibernateException;
import org.hibernate.Session;

import com.softslate.commerce.businessobjects.shipping.ShippingRule;
import com.softslate.commerce.daos.core.BaseDAO;
import com.softslate.commerce.daos.core.DAOInterface;
import com.softslate.commerce.daos.core.DataAccessException;

/**
 * Generic DAO to manage the <code>sscShippingRule</code> table of the database.
 * 
 * @author Jason McWilliams
 */
public class ShippingRuleDAOHibernate extends BaseDAO implements DAOInterface,
        ShippingRuleDAO {

    static Log log = LogFactory.getLog(ShippingRuleDAOHibernate.class);

    private ShippingRule shippingRule = null;

    public ShippingRule getShippingRule() {
        return (this.shippingRule);
    }

    public void setShippingRule(ShippingRule shippingRule) {
        this.shippingRule = shippingRule;
    }

    public void loadShippingRuleFromID() throws Exception {
        Session session = getDaoFactory().startSession();
        try {
            ShippingRule shippingRule = (ShippingRule) session.get(ShippingRule.class,
                    new Integer(getShippingRule().getShippingRuleID()));
            PropertyUtils.copyProperties(getShippingRule(), shippingRule);
            setShippingRule(shippingRule);
        } catch (HibernateException he) {
            if (log.isErrorEnabled())
                log.error("Hibernate error: " + he.toString());
            throw new DataAccessException(he);
        }
        return;
    }

    public void insertShippingRule() throws DataAccessException {
        insertShippingRule(false);
    }

    public void insertShippingRule(boolean commit) throws DataAccessException {
        Session session = getDaoFactory().startSession();
        try {
            session.save(getShippingRule());
            if (commit) {
                getDaoFactory().commitTransaction();
            }
            evictCache();
        } catch (HibernateException he) {
            if (log.isErrorEnabled())
                log.error("Hibernate error: " + he.toString());
            throw new DataAccessException(he);
        }
        return;
    }

    public void updateShippingRule() throws DataAccessException {
        updateShippingRule(false);
    }

    public void updateShippingRule(boolean commit) throws DataAccessException {
        Session session = getDaoFactory().startSession();
        try {
            session.update(getShippingRule());
            if (commit) {
                getDaoFactory().commitTransaction();
            }
        } catch (HibernateException he) {
            if (log.isErrorEnabled())
                log.error("Hibernate error: " + he.toString());
            throw new DataAccessException(he);
        }
        return;
    }

    public void deleteShippingRule() throws DataAccessException {
        deleteShippingRule(false);
    }

    public void deleteShippingRule(boolean commit) throws DataAccessException {
        Session session = getDaoFactory().startSession();
        try {
            setShippingRule((ShippingRule) session.load(ShippingRule.class, new Integer(
                    getShippingRule().getShippingRuleID())));
            session.delete(getShippingRule());
            if (commit) {
                getDaoFactory().commitTransaction();
            }
            evictCache();
        } catch (HibernateException he) {
            if (log.isErrorEnabled())
                log.error("Hibernate error: " + he.toString());
            throw new DataAccessException(he);
        }
        return;
    }
}
