package com.softslate.commerce.installer;

import java.io.File;
import java.io.FileOutputStream;
import java.util.Properties;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionErrors;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.action.ActionMessage;
import org.apache.struts.action.ActionMessages;
import org.apache.struts.action.DynaActionForm;

import com.softslate.commerce.customer.core.AppUtils;
import com.softslate.commerce.customer.core.BaseAction;
import com.softslate.commerce.customer.core.ConfigPlugIn;
import com.softslate.commerce.daos.core.DAOFactory;

/**
 * Save entered settings for database connectivity to properties files, reload
 * the Hibernate SessionFactory, and optionally test the the database
 * connection.
 * 
 * @author David Tobey
 */
public class DBSettingsAction extends BaseAction {

	static Log log = LogFactory.getLog(DBSettingsAction.class);

	public ActionForward execute(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response)
			throws Exception {
		if (log.isDebugEnabled())
			log.debug("Starting action.");
		ActionErrors errors = new ActionErrors();
		ActionMessages messages = new ActionMessages();
		DynaActionForm dynaForm = (DynaActionForm) form;
		boolean error = false;

		// Initialize some incoming settings.
		String dialect = (String) dynaForm.get("dialect");
		String otherType = (String) dynaForm.get("otherType");
		if (otherType != null && otherType.length() > 0) {
			dialect = otherType;
		}
		String databaseDriver = (String) dynaForm.get("databaseDriver");
		String otherDriver = (String) dynaForm.get("otherDriver");
		if (otherDriver != null && otherDriver.length() > 0) {
			databaseDriver = otherDriver;
		}
		String databaseURL = (String) dynaForm.get("databaseURL");
		String otherURL = (String) dynaForm.get("otherURL");
		if (otherURL != null && otherURL.length() > 0) {
			databaseURL = otherURL;
		}

		// Save the hibernate settings to hibernate.properties
		String hibernatePropertiesFile = "hibernate.properties";
		if (System.getProperty("hibernate.configuration") != null) {
			hibernatePropertiesFile = System
					.getProperty("hibernate.configuration");
		}
		String hibSettingsPath = getServlet().getServletContext().getRealPath(
				"/WEB-INF/classes/" + hibernatePropertiesFile);
		if (hibSettingsPath == null) {
			errors.add(ActionMessages.GLOBAL_MESSAGE, new ActionMessage(
					"installer.warError", hibSettingsPath));
			error = true;
		} else {
			try {
				Properties hibSettings = null;
				if (log.isDebugEnabled())
					log
							.debug("Save the hibernate settings to " + hibernatePropertiesFile);
				synchronized (getServlet().getServletContext()) {
					File hibSettingsFile = new File(hibSettingsPath);
					hibSettings = AppUtils.loadProperties(hibSettingsFile);
					hibSettings.setProperty("hibernate.dialect", dialect);
					hibSettings
							.setProperty("hibernate.connection.driver_class",
									databaseDriver);
					hibSettings.setProperty("hibernate.connection.url",
							databaseURL);
					hibSettings.setProperty("hibernate.connection.username",
							(String) dynaForm.get("databaseUserName"));
					hibSettings.setProperty("hibernate.connection.password",
							(String) dynaForm.get("databasePassword"));
					hibSettings.setProperty("hibernate.listeners.envers.autoRegister",
							(String) dynaForm.get("enableEnversAuditLogging"));
					if (log.isDebugEnabled())
						log.debug("Attempting to write settings to "
								+ hibSettingsPath + " hibSettings: "
								+ hibSettings);
					FileOutputStream fos2 = new FileOutputStream(
							hibSettingsFile);
					hibSettings.store(fos2, "Generated by installer.");
				}
				messages.add(ActionMessages.GLOBAL_MESSAGE, new ActionMessage(
						"installer.dbSettingsSaved", hibSettingsPath));
				
				// In case we're in an environment with an src folder, save
				// the file to the src area too.
				String hibSettingsPathSrc = getServlet().getServletContext()
						.getRealPath("/WEB-INF/src/" + hibernatePropertiesFile);
				File hibSettingsFileSrc = new File(hibSettingsPathSrc);
				if (hibSettings != null && hibSettingsFileSrc.exists()) {
					FileOutputStream fos3 = new FileOutputStream(
							hibSettingsFileSrc);
					hibSettings.store(fos3, "Generated by installer.");
					messages.add(ActionMessages.GLOBAL_MESSAGE,
							new ActionMessage("installer.dbSettingsSaved",
									hibSettingsPathSrc));
				}
			} catch (Exception fnfe) {
				log.error(fnfe.toString(), fnfe);
				errors.add(ActionMessages.GLOBAL_MESSAGE, new ActionMessage(
						"installer.fileNotFoundError", hibSettingsPath));
				error = true;
			}
		}

		// Attempt to get a connection.
		String testConnection = (String) dynaForm.get("testConnection");
		boolean connectedToDatabase = false;
		DAOFactory daoFactory = null;
		String errorMessage = "";
		if (testConnection != null && testConnection.equals("true")) {
			try {
				// Reinitialize the Hibernate SessionFactory with the new values
				ConfigPlugIn configPlugIn = new ConfigPlugIn(servlet);
				configPlugIn.initializeDAOFactory();
				daoFactory = (DAOFactory) servlet.getServletContext()
						.getAttribute("daoFactory");
				if (log.isDebugEnabled())
					log.debug("Testing connection.");
				if (daoFactory != null) {
					daoFactory.startSession();
					connectedToDatabase = daoFactory.getSession().isConnected();
				}
			} catch (Exception sqle) {
				log.error("Connection failed: " + sqle.toString(), sqle);
				errorMessage = sqle.toString();
			}
		}
		if (connectedToDatabase) {
			messages.add(ActionMessages.GLOBAL_MESSAGE, new ActionMessage(
					"installer.testConnectionMade", daoFactory
							.getDatabaseType()));

		} else {
			errors.add(ActionMessages.GLOBAL_MESSAGE, new ActionMessage(
					"installer.connectionTestFailed", errorMessage));
			error = true;
		}
		if (!messages.isEmpty())
			addMessages(request, messages);
		if (!errors.isEmpty())
			addErrors(request, errors);
		if (log.isDebugEnabled())
			log.debug("Finished with Action.");
		if (error) {
			return mapping.findForward("error");
		}
		return mapping.findForward("success");
	}
}
